@@ -116,7 +116,7 @@ class TestCreateNewNounData(unittest.TestCase):
         # create copy and reload module:
         old_gender_dict = copy.deepcopy(gn.GENDER_DICT)
         pipeline_output = GenderNounDataHandler.create_full_graph_from_web()
-        os.remove("src/gendered-nouns.gdn")
+        os.remove("src/data/gendered-nouns.gdn")
         with self.assertWarns(ws.GenderedNounsBuildFromWebWarning):
             importlib.reload(sys.modules["src.gender_nouns"])
 
@@ -282,9 +282,9 @@ class TestGenderNounDataHandler(unittest.TestCase):
         inp = {"wuwu": {"warning": ["warning1", "warning2"]}}
         out = {"wuwu": {"warning": {"warning1", "warning2"}}}
         with open("test.gdn", "w") as test_file:
-            test_file.write(json.dumps(inp))
+            test_file.write(json.dumps({"data": inp}))
         # test if it loads as expected:
-        self.assertEqual(GenderNounDataHandler.load_from_disk("test.gdn"), out)
+        self.assertEqual(GenderNounDataHandler.load_from_disk("test.gdn"), (out, dict()))
 
         # test for dict without warning:
 
@@ -292,9 +292,9 @@ class TestGenderNounDataHandler(unittest.TestCase):
         inp = {"wuwu": {"fufu": "wawa"}}
         out = inp
         with open("test.gdn", "w") as test_file:
-            test_file.write(json.dumps(inp))
+            test_file.write(json.dumps({"data": inp}))
         # test if it loads directly:
-        self.assertEqual(GenderNounDataHandler.load_from_disk("test.gdn"), out)
+        self.assertEqual(GenderNounDataHandler.load_from_disk("test.gdn"), (out, dict()))
 
         # test for a mixture:
 
@@ -302,9 +302,9 @@ class TestGenderNounDataHandler(unittest.TestCase):
         inp = {"wuwu": {"warning": ["warning1", "warning2"]}, "wawa": {"fufu": "wawa"}}
         out = {"wuwu": {"warning": {"warning1", "warning2"}}, "wawa": {"fufu": "wawa"}}
         with open("test.gdn", "w") as test_file:
-            test_file.write(json.dumps(inp))
+            test_file.write(json.dumps({"data": inp}))
         # test if it loads as expected:
-        self.assertEqual(GenderNounDataHandler.load_from_disk("test.gdn"), out)
+        self.assertEqual(GenderNounDataHandler.load_from_disk("test.gdn"), (out, dict()))
 
         # finally delete the file:
         os.remove("test.gdn")
@@ -317,7 +317,7 @@ class TestGenderNounDataHandler(unittest.TestCase):
         data_loaded = {"wuwu": {"warning": {"warning1", "warning2"}}}
         GenderNounDataHandler.save_to_disk(data_loaded, "test.gdn")
         with open("test.gdn", "r") as test_file:
-            self.assertEqual(json.loads(test_file.read()), data_in_file)
+            self.assertEqual(json.loads(test_file.read()), {"data": data_in_file})
 
         # test for dict without warning:
 
@@ -326,7 +326,7 @@ class TestGenderNounDataHandler(unittest.TestCase):
         data_loaded = data_in_file
         GenderNounDataHandler.save_to_disk(data_loaded, "test.gdn")
         with open("test.gdn", "r") as test_file:
-            self.assertEqual(json.loads(test_file.read()), data_in_file)
+            self.assertEqual(json.loads(test_file.read()), {"data": data_in_file})
 
         # test for a mixture:
 
@@ -335,7 +335,7 @@ class TestGenderNounDataHandler(unittest.TestCase):
         data_loaded = {"wuwu": {"warning": {"warning1", "warning2"}}, "wawa": {"fufu": "wawa"}}
         GenderNounDataHandler.save_to_disk(data_loaded, "test.gdn")
         with open("test.gdn", "r") as test_file:
-            self.assertEqual(json.loads(test_file.read()), data_in_file)
+            self.assertEqual(json.loads(test_file.read()), {"data": data_in_file})
 
         # finally delete the file:
         os.remove("test.gdn")
@@ -346,26 +346,32 @@ class TestGenderNounDataHandler(unittest.TestCase):
         # save a file to disk to test it:
         data = {"wuwu": {"warning": {"warning1", "warning2"}}}
         GenderNounDataHandler.save_to_disk(data, "test.gdn")
-        self.assertEqual(data, GenderNounDataHandler.load_from_disk("test.gdn"))
+        self.assertEqual((data, dict()), GenderNounDataHandler.load_from_disk("test.gdn"))
 
         # test for dict without warning:
 
         # save a file to disk to test it:
         data = {"wuwu": {"fufu": "wawa"}}
         GenderNounDataHandler.save_to_disk(data, "test.gdn")
-        self.assertEqual(data, GenderNounDataHandler.load_from_disk("test.gdn"))
+        self.assertEqual((data, dict()), GenderNounDataHandler.load_from_disk("test.gdn"))
 
         # test for words with underscores:
         data = {"fu_fu": {"wawa": "wuwu"}}
         GenderNounDataHandler.save_to_disk(data, "test.gdn")
-        self.assertEqual(data, GenderNounDataHandler.load_from_disk("test.gdn"))
+        self.assertEqual((data, dict()), GenderNounDataHandler.load_from_disk("test.gdn"))
 
         # test for a mixture:
 
         # save a file to disk to test it:
         data = {"wuwu": {"warning": {"warning1", "warning2"}}, "wa_wa": {"fufu": "wawa"}}
         GenderNounDataHandler.save_to_disk(data, "test.gdn")
-        self.assertEqual(data, GenderNounDataHandler.load_from_disk("test.gdn"))
+        self.assertEqual((data, dict()), GenderNounDataHandler.load_from_disk("test.gdn"))
+
+        # test for data with meta data:
+        data = {"wuwu": {"warning": {"warning1", "warning2"}}, "wa_wa": {"fufu": "wawa"}}
+        meta_data = {"foo": "bar", "a": "b"}
+        GenderNounDataHandler.save_to_disk(data, "test.gdn", **meta_data)
+        self.assertEqual((data, meta_data), GenderNounDataHandler.load_from_disk("test.gdn"))
 
         # finally delete the file:
         os.remove("test.gdn")
@@ -784,18 +790,15 @@ class TestGenderedNoun(unittest.TestCase):
             n.render_noun("neutral")
             self.assertEqual(w, [])
 
-        # make sure the word is saved in the right uppercase/lowercase form, but warnings are risen based on its
-        # lowercase form:
-        with warnings.catch_warnings(record=True) as w:
-            n = gn.GenderedNoun("Bigot")
-            self.assertEqual(w, [])
-        self.assertEqual(n.word, "Bigot")
+        # make sure we correctly removed the capitalization code from the code for nouns gendering:
+        with self.assertWarns(ws.NotAPersonNounWarning):
+            n = gn.GenderedNoun("Bishop")
+        self.assertEqual(n.word, "Bishop")
 
     def test_render_noun(self):
         with warnings.catch_warnings(record=True) as w:
             # every test case refers to a specific type of lookup-scenario; and every test case has ana additional test
-            # case for the scenario that the resulting word contains underscores attached, as well as one for the
-            # scenario that the given word is uppercase:
+            # case for the scenario that the resulting word contains underscores attached:
 
             # test for a non-existent version of a neutral word (careerist is neutral and has no male version, so it
             # should instead give us itself):
@@ -804,8 +807,6 @@ class TestGenderedNoun(unittest.TestCase):
             self.assertEqual(gn.GenderedNoun("careerist").render_noun("male"), "careerist")
             # correctly replace underscores with whitespace in this scenario:
             self.assertEqual(gn.GenderedNoun("big_spender").render_noun("male"), "big spender")
-            # uppercase:
-            self.assertEqual(gn.GenderedNoun("Careerist").render_noun("male"), "Careerist")
 
             # test for non-existent version of word with neutral version ("townee" is male, has a neutral version, but
             # no female version, so if we request the female version of it, we should get its neutral version):
@@ -814,23 +815,17 @@ class TestGenderedNoun(unittest.TestCase):
             self.assertEqual(gn.GenderedNoun("townee").render_noun("female"), "townee")
             # correctly replace underscores with whitespace in this scenario:
             self.assertEqual(gn.GenderedNoun("closet_queen").render_noun("female"), "closet monarch")
-            # uppercase:
-            self.assertEqual(gn.GenderedNoun("Townee").render_noun("female"), "Townee")
 
             # test for word that is already the right gender (tests for both neutral as well as non-neutral gender):
             self.assertEqual(gn.GENDER_DICT["townsman"]["gender"], "male")
             self.assertEqual(gn.GenderedNoun("townsman").render_noun("male"), "townsman")
             # correctly replace underscores with whitespace in this scenario:
             self.assertEqual(gn.GenderedNoun("sea_scout").render_noun("female"), "sea scout")
-            # uppercase:
-            self.assertEqual(gn.GenderedNoun("Townsman").render_noun("male"), "Townsman")
 
             self.assertEqual(gn.GENDER_DICT["tourist"]["gender"], "neutral")
             self.assertEqual(gn.GenderedNoun("tourist").render_noun("neutral"), "tourist")
             # correctly replace underscores with whitespace in this scenario:
             self.assertEqual(gn.GenderedNoun("second_cousin").render_noun("female"), "second cousin")
-            # uppercase:
-            self.assertEqual(gn.GenderedNoun("Tourist").render_noun("neutral"), "Tourist")
 
             # return correctly gendered version for words from the gender_map, if word is not the gender it requests and
             #  has the gender in its gender map (test for requesting a non-neutral as well as the neutral version):
@@ -839,13 +834,9 @@ class TestGenderedNoun(unittest.TestCase):
             self.assertEqual(gn.GenderedNoun("big_brother").render_noun("female"), "big sister")
             # correctly replace underscores with whitespace in this scenario:
             self.assertEqual(gn.GenderedNoun("ring_girl").render_noun("male"), "ring boy")
-            # uppercase:
-            self.assertEqual(gn.GenderedNoun("Big_brother").render_noun("female"), "Big sister")
 
             self.assertEqual(set(gn.GENDER_DICT["black_man"]["gender_map"].keys()), {"female", "neutral"})
             self.assertEqual(gn.GENDER_DICT["black_man"]["gender"], "male")
             self.assertEqual(gn.GenderedNoun("black_man").render_noun("neutral"), "black person")
             # correctly replace underscores with whitespace in this scenario:
             self.assertEqual(gn.GenderedNoun("ring_girl").render_noun("neutral"), "ring bean")
-            # uppercase:
-            self.assertEqual(gn.GenderedNoun("Black_man").render_noun("neutral"), "Black person")
